package drakcluster::actions;

use strict;

use common;
use ugtk2 qw(:wrappers :create);
use interface_cluster;
use dns_cluster;
use maui_cluster;
use server_cluster;
use dhcpnode_cluster;
use cluster_fonction_common;
use cluster_set_admin;

our @ISA = qw(Exporter);
our @EXPORT = qw(run_command_and_log gtktext_get_log);

my @actions = (
    [ N("Run")             => 'has_any',   \&action_run ],
    [ N("Power Off")       => 'has_any',   sub { action_poweroff('poweroff', @_) } ],
    [ N("Power On")        => 'has_any',   sub { action_poweron('poweron', @_) } ],
    [ N("Reboot")          => 'has_any',   sub { action_reboot('reboot', @_) } ],
    [ N("Install\nPackage") => 'has_group', sub { action_package('install', @_) } ],
    [ N("Remove\nPackage")  => 'has_group', sub { action_package('remove', @_) } ],
);

my %popup = (
    name => [
        [ N("Replicate Node"), sub { 
	      my ($_l, $_l_current, $current, $_categories, $_nodes, $update_node) = @_;
	      my ($nb, $device, $yn) = (1, '', 0); my $skip;
	      my $max_nodes = cluster_serverconf::cluster_data()->{FINISHNODE} - cluster_serverconf::cluster_data()->{STARTNODE} - interface_cluster::get_number_of_nodes() + 1;
	      $max_nodes > 0 or $::in->ask_warn('', N("You've reach the maximum number of nodes")), return;
	      $::in->ask_from('', N("Using node \"%s\" as the golden node.", $current),
			      [ { label => N("Number of nodes to add"), val => \$nb, type => 'range', min => 1, max => $max_nodes },
				{ label => N("Skip dhcpd detection ? (node must be in dhcpd.conf)"), val => \$yn, type => 'bool', advanced => 1 },
			        { label => N("\n\nHow many nodes do you want to install by using %s as the golden node. By default the cloning process use the kamethod, if you wan to use the dolly one, do it by hand (see the documentation).\n", $current), advanced => 1 },
				{ label => N("Hard drive device to duplicate (hda, sda, cciss...)"), val => \$device },
				{ label => N("\nhda -> IDE hard drive disk\nsda -> scsi hard drivedisk\ncciss -> compaq smart array"),  },
			      ],
			      complete => sub {
				$device or $::in->ask_warn('', N("Please enter a valid device")), return 1,1;
				0;
			      }) or return $yn;
	      if ($yn =~ /1/) { $skip = "-s" }
	      warn "Adding $nb nodes using $current as golden node and $device as hard drive device.\n";
	      run_command_and_log(interface_cluster::add_nodes($nb, $current, $device, $skip), N("Adding %d nodes using %s as golden node and %s as device", $nb, $current, $device, $skip), sub { $update_node->('add') });
	      'add'; # action "add" is delayed
	  } ],
        [ N("Remove Node"), sub { 
	      my ($_l, $l_current) = @_;
	      $::in->ask_okcancel('', N("Do you really want to remove the following nodes %s", join(', ', @$l_current))) or return;
	      warn "Removing nodes ", join(', ', @$l_current), "\n";
              my $comment = "Removing " . join(', ', @$l_current) . " from cluster";
              my $_w = $::in->wait_message('', $comment . '...');
              interface_cluster::remove_nodes($l_current);
	      'remove';
          } ],
     [ N("Reload Configuration"), sub { 
		 my ($_l, $l_current) = @_;
		 my $n = join(" -m ", @$l_current);
		 run_command_and_log("rshp2 -v -c ssh -m $n -- /usr/bin/setup_client_cluster.pl reload_c", N("updating client nodes %s", join(', ', @$l_current)));
		 'reload';
       } ],
     [ N("Open Terminal"), sub { 
		 my ($_l, $l_current) = @_;
                 foreach (@$l_current) {
		   system("nohup xterm -geometry 40x12 -fn 5x9 -e ssh $_ &");
		 }
		 'remote';
       } ],
    ],
    cpu => [
	    [ N("Get number of CPU(s) "), sub {
		run_command_and_log(("setup_recup_cpus.pl"), N("updating number of CPU on ALL client nodes"));
		'test';
	      } ],
    ],
	     dhcp => [
		      [ N("Add nodes in dhcpd.conf"), sub {
			  my $max_nodes = cluster_serverconf::cluster_data()->{FINISHNODE} - cluster_serverconf::cluster_data()->{STARTNODE} - interface_cluster::get_number_of_nodes() + 1;
			  my $nb;
			  $::in->ask_from('', N("Add a node in dhcpd.conf"),
					  [ { label => N("Number of nodes to add"), val => \$nb, type => 'range', min => 1, max => $max_nodes },
					  ],
					 ) or return;
			  my $comment = "Add $nb nodes in dhcpd.conf";
			  my $_w = $::in->wait_message('', $comment . '...');
			  run_command_and_log(("setup_add_nodes_to_dhcp.pl -n $nb"), N("adding a node in dhcpd.com"));
			  'changedhcp'; # action "add" is delayed
			} ],
		      [ N("Show nodes MAC address"), sub {
			  run_command_and_log(("dhcpnode -l"), N("Nodes MAC address"));
			  'changedhcp';
			} ],
		     ],
	     partition => [
        [ N("Change node to another MAUI partition"), sub { 
	      my ($l, $l_current, $val) = @_;
	      $::in->ask_from('', N("Please choose the computing partition for nodes %s?", join(', ', @$l_current)),
			      [ { val => \$val, list => [ '', @$l ] } ],
			      complete => sub {
				  $val or $::in->ask_warn('', N("Please enter a valid partition")), return 1;
				  0;
			      }) or return $val;
	      my $comment = "Updating nodes maui partition";
	      warn $comment . ".\n";
	      my $_w = $::in->wait_message('', $comment . '...');
	      interface_cluster::chg_node_part($_, $val) foreach @$l_current;
	      service_do('maui', 'restart');
	      'change', $val;
          } ],

	[ N("Add new MAUI partition"), sub { 
	      my ($l, $_l_current, $_val) = @_;
	      my $part;
	      $::in->ask_from('', N("Please enter the the name of new computing partition you want to create."),
			      [ { val => \$part } ],
			      complete => sub {
				  member($part, @$l) and $::in->ask_warn('', N("Partition %s already exists", $part)), return 1;
				  $part or $::in->ask_warn('', N("Please add a name for this new partition", $part)), return 1;
				  0;
			      }) && $part or return;
	      my $comment = "add $part in maui configuration";
	      my $_w = $::in->wait_message('', $comment . '...');
	      push @$l, $part;
	      maui_cluster::add_partition($part);
	      'change partitions'
	  } ],
        [ N("Remove a MAUI partition"), sub {
	      my ($l, $_l_current, $_val, $categories, $nodes) = @_;
	      my $part;
	      $::in->ask_from('', N("Please choose the computing partition you want to remove"),
			      [ { val => \$part, list => $l } ],
			      complete => sub {
				  (any { $_->{partition} eq $part } values %$nodes) || (any { $_->{$part} } values %{$categories->{users}}) and $::in->ask_warn('', N("Partition %s is still in use, you can't remove it")), return 1;
				  0;
			      }) or return;
	      my $comment = "remove $part in maui configuration";
	      my $_w = $::in->wait_message('', $comment . '...');
	      @$l = grep { $_ ne $part } @$l;
	      maui_cluster::remove_partition($part);
	      service_do('maui', 'restart');
	      'change partitions'
	  } ],
    ],
    group => [
	      [ N("Change node to another urpmi Group"), sub {
		  my ($l, $l_current, $val) = @_;
		  $::in->ask_from('', N("Select the urpmi group for nodes %s?", join(', ', @$l_current)),
				  [ { val => \$val, list => [ '', @$l ] } ],
				  complete => sub {
				      $val or $::in->ask_warn('', N("Please enter a valid group")), return 1;
				      0;
				  }) or return $val;
		  cluster_set_admin::urpmipp_chg_node_group($_, $val) foreach @$l_current;
		  'change', $val;
	      } ],
	      [ N("Add a new urpmi Group"), sub {
		  my ($l, $l_current, $val) = @_;
		  my $ugroup;
		  $::in->ask_from('', N("Name of the new urpmi group you want to create:"),
				  [ { val => \$ugroup } ],
				  complete => sub {
				      member($ugroup, @$l) and $::in->ask_warn('', N("Partition %s already exists", $ugroup)), return 1;
				      $ugroup or $::in->ask_warn('', N("Please add a name for this new urpmi group", $ugroup)), return 1;
				      0;
				  }) && $ugroup or return;
		  push @$l, $ugroup;
		  cluster_set_admin::urpmipp_add_group($ugroup);
		  'changeg'
		  } ],
        [ N("Remove an urpmi Group"), sub {
	      my ($l, $_l_current, $_val) = @_;
	      my $ugroup;
	      $::in->ask_from('', N("Please choose the urpmi group you want to remove:"),
			      [ { val => \$ugroup, list => $l } ],
			      complete => sub {
				if (any { /$ugroup:ka-run:-c ssh -m/ } cat_(cluster_serverconf::urpmi_data()->{URPMICFG})) {
				  $::in->ask_warn('', N("Urpmi group %s already in use, cant remove it", $ugroup)), return 1;
				} else {
				  0;
				}
			      }) && $ugroup or return;
	      pop @$l, $ugroup;
	      cluster_set_admin::urpmipp_remove_group($ugroup);
	      'changeg'
	    } ],
    ],
   state => [
	     [ N("Restart gmond"), sub {
		 my ($_l, $l_current) = @_;
                 my $n = join(" -m ", @$l_current);
                 run_command_and_log("rshp2 -v -c ssh -m $n -- service gmond restart", N("restarting gmond %s", join(', ', @$l_current)));
                 'gmond';
	       }, ],
	     [ N("Restart Network"), sub {
		 my ($_l, $l_current) = @_;
                 my $n = join(" -m ", @$l_current);
                 run_command_and_log("rshp2 -v -c ssh -m $n -- service network restart", N("restarting network %s", join(', ', @$l_current)));
                 'network';
	       }, ],
	    ],
	     oar => [
		     [ N("OAR nodes status"), sub {
			 run_command_and_log("oarnodes -s", N("OAR nodes status"), "0");
			 'oar';
		       }, ],
		     [ "",  sub { } ],
		     [ N("Alive state in OAR"), sub {
			 my ($_l, $l_current) = @_;
			 $::in->ask_okcancel('', N("Do you really want to set the following nodes %s in Alive state in OAR?", join(', ', @$l_current))) or return;
			 warn "Setting nodes in Alive state", join(', ', @$l_current), "\n";
			 my $comment = "Setting " . join(', ', @$l_current) . " state Alive";
			 my $_w = $::in->wait_message('', $comment . '...');
			 interface_cluster::change_oar_state($l_current, "Alive");
                         'changeoar';
                       }, ],
		     [ N("Absent state in OAR"), sub {
			 my ($_l, $l_current) = @_;
			 $::in->ask_okcancel('', N("Do you really want to set the following nodes %s in Absent state in OAR?", join(', ', @$l_current))) or return;
			 warn "Setting nodes in Absent state", join(', ', @$l_current), "\n";
			 my $comment = "Setting " . join(', ', @$l_current) . " state Absent";
			 my $_w = $::in->wait_message('', $comment . '...');
			 interface_cluster::change_oar_state($l_current, "Absent");
                         'changeoar';
                       }, ],
		     [ N("Dead state in OAR"), sub {
			 my ($_l, $l_current) = @_;
			 $::in->ask_okcancel('', N("Do you really want to set the following nodes %s in Dead state in OAR?", join(', ', @$l_current))) or return;
			 warn "Setting nodes in Dead state", join(', ', @$l_current), "\n";
			 my $comment = "Setting " . join(', ', @$l_current) . " state Dead";
			 my $_w = $::in->wait_message('', $comment . '...');
			 interface_cluster::change_oar_state($l_current, "Dead");
                         'changeoar';
                       }, ],

		     [ "",  sub { } ],
                     [ N("Remove a node from OAR"), sub {
			 my ($_l, $l_current) = @_;
			 $::in->ask_okcancel('', N("Do you really want to remove the following nodes %s from OAR node list?", join(', ', @$l_current))) or return;
			 warn "Removing nodes from OAR configuration", join(', ', @$l_current), "\n";
			 my $comment = "Removing " . join(', ', @$l_current) . " from OAR configuration";
			 my $_w = $::in->wait_message('', $comment . '...');
			 interface_cluster::remove_nodes_in_oar($l_current);
			 'changeoar';
                       }, ],
		    ],
);

sub one_popup_menu {
    my ($column_name, $categories, $nodes, $click_time, $l_current, $val, $update_node) = @_;
    my $menu_treeview = Gtk2::Menu->new;
    foreach (@{$popup{$column_name}}) {
	my ($descr, $f) = @$_;
	gtkappend($menu_treeview,
		  gtksignal_connect(Gtk2::MenuItem->new_with_label($descr), 
				    activate => sub {
					my @r = $f->($categories->{$column_name}, $l_current, $val, $categories, $nodes, $update_node);
					$update_node->(@r);
				    }));
    }
    $menu_treeview->popup(undef, undef, undef, undef, '3', $click_time);
}

sub popup_menus {
    my ($categories) = @_;
    my %l = map { 
	my $column_name = $_;
	$_ => sub { one_popup_menu($column_name, $categories, @_) };
    } keys %popup;
    \%l;
}

sub commands_list() {
    my @l = (
	     [ N("Setup the Server Installer cluster   (nfs, ftp, copy cdrom)") => 'setup_install_cluster.pl doall' ],
	     [ N("Setup master DNS server   (resolv.conf, rndc.key, hosts, domain)") => 'setup_dns.pl master' ],
	     [ N("Setup cluster dhcpd configuration   (dhcpd.conf)") => 'setup_dhcpdconf_server.pl doall' ],
	     [ N("Setup NIS server with Autofs   (autofs map, nis domain, yp)") => 'setup_nis.pl set_server' ],
	     [ N("Build PXE images to control Client boot nodes   (local linux linux-nofb freedos ka)") => 'setup_pxe_server.pl build' ],
	     [ N("Setup hdlists   (regenerate hdlist from install directory)") => 'setup_hdlists_server.pl generate' ],
	     [ N("Setup Postfix server   (domain, origin)") => 'setup_postfix.pl set' ],
	     [ N("Setup the server   (ganglia, nis for tftp, rsh, ssh, authd, xdm, service, nodes list)") => 'setup_server_cluster.pl doall' ],
	     [ N("Setup administration   (clusterit, gexec, rshp2, urpmi parallel)") => 'setup_admin.pl doall' ],
	     [ N("Setup compute   (pbs node list, distcc, mpi, lam, maui)") => 'setup_compute.pl doall' ],
	     [ N("Setup PBS   (maui integration, queue, xpbs, configuration file)") => 'setup_pbs.pl doall' ],
	     [ N("Setup MAUI   (nodes, partition, maui user)") => 'setup_maui.pl doall' ],
	     if_(dns_cluster::check_same_ip(),
		 [ N("Activate compute interface (add compute interface in dhcpd.conf)") => 'setup_add_nodes_to_dhcp.pl -c' ]
		),
	    );
    map {
	my ($txt, $cmd) = @$_;
	[ $txt, sub {
	    $::in->ask_yesorno('', N("Run this script will reset\n \"$txt\" configuration\n \n Are You sure ?")) or return;
	    run_command_and_log($cmd, $txt);
	  } ];
    } @l;
}

sub create_buttons {
    my ($selection) = @_;

    my $actions_box = Gtk2::HButtonBox->new;

    my @buttons_set_sensitive = map {
	my ($descr, $flag, $f) = @$_;

	my $button = Gtk2::Button->new($descr);
	$button->signal_connect(clicked => sub { $f->($selection, $descr) });
	gtkadd($actions_box, $button);
	sub { $button->set_sensitive(member($flag, @_)) }
    } @actions;

    $actions_box, @buttons_set_sensitive;
}

sub selection_to_string {
    my ($selection) = @_;
    @$selection > 10 ? N("the %d selected nodes", int @$selection) : N("nodes %s", join(", ", map { $_->{name} } @$selection));
}

sub selection_to_string_on {
    my ($selection) = @_;
    @$selection > 10 ? N("on the %d selected nodes", int @$selection) : N("on nodes %s", join(", ", map { $_->{name} } @$selection));
}

sub action_simple {
    my ($action, $selection, $descr) = @_;
    $::in->ask_okcancel('', N("Really \"%s\" %s?", $descr, selection_to_string($selection))) or return;
    warn "call $action on ", selection_to_string($selection), "\n";
    1;
}

sub action_poweron {
  my ($action, $selection, $descr) = @_;
  if (action_simple($action, $selection, $descr)) {
    foreach my $node (@$selection) {
      interface_cluster::cmd_poweron($node->{name});
    }
  }
}

sub action_poweroff {
  my ($action, $selection, $descr) = @_;
  my $list;
  if (action_simple($action, $selection, $descr)) {
    foreach my $node (@$selection) {
      $list = $list . " -w " . $node->{name};
    }
    interface_cluster::cmd_poweroff($list);
  }
}

sub action_reboot {
  my ($action, $selection, $descr) = @_;
  my $list;
  if (action_simple($action, $selection, $descr)) {
    foreach my $node (@$selection) {
      $list = $list . " -w " . $node->{name};
    }
    interface_cluster::cmd_reboot($list);
  }
}

sub action_package {
    my ($action, $selection, $descr) = @_;
    my $pkgs = $::in->ask_from_entry($descr, 
				     $action eq 'install' ? 
				       N("Which package(s) do you want to install %s", selection_to_string_on($selection)) : 
				       N("Which package(s) do you want to remove %s", selection_to_string_on($selection))) or return;
    my @groups = uniq(map { $_->{group} } @$selection);
    warn "$action packages $pkgs on @groups\n";
    run_command_and_log($action eq 'install' ? 
			(interface_cluster::cmd_install_pkg(\@groups, $pkgs), N("Installing packages %s", $pkgs)) : 
			(interface_cluster::cmd_remove_pkg(\@groups, $pkgs), N("Removing packages %s", $pkgs)));
}

my $timeout;
sub gtktext_get_log {
    my ($command, $log_w, $log_scroll, $when_command_is_over) = @_;

    my $buffer = $log_w->get_buffer;
    $buffer->delete($buffer->get_start_iter, $buffer->get_end_iter);

    my ($prev_scroll, $want_scroll_down) = (0, 1);

    my $pid = open(my $F, "$command 2>&1 |") or return sub { 1 };

    fcntl($F, c::F_SETFL(), c::O_NONBLOCK()) or die "can't fcntl F_SETFL: $!";

    #Glib::Source->remove($timeout) if $timeout; $timeout = '';
    $timeout = Glib::Timeout->add(100, sub {
        if ($buffer) {
	    my $end = $buffer->get_end_iter;
	    while (defined (my $s = <$F>)) {
		$buffer->insert_interactive($end, $s, -1);
	    }
	    my $new_scroll = $log_scroll->child->get_vadjustment->get_value;
	    $want_scroll_down &&= $new_scroll >= $prev_scroll;
	    $prev_scroll = $new_scroll;
	    $log_w->scroll_to_iter($end, 0, 0, 0, 0) if $want_scroll_down;
	}
	if (waitpid($pid, c::WNOHANG()) > 0) {
	    #$buffer->insert($buffer->get_end_iter, "\n" . "-" x 90 . "\n" . "*-end-*" x 10, -1) if $buffer;
	    $buffer->insert_interactive($buffer->get_end_iter, "\n" . "#" x 90 . "\n" . " This script has terminated, you can close this window" . "\n" . "#" x 90, -1) if $buffer;
	    $when_command_is_over->() if $when_command_is_over;
	    $pid = $timeout = '';
	    0;
	} else {
	    1;
	}
    });
    sub {
	if ($pid) {
	    $::in->ask_yesorno('', N("The command is still running. Do you want to kill it?")) or return;
	    kill 9, $pid+1;
	}
	$buffer = ''; 
	1;
    };
}

sub run_command_and_log {
    my ($command, $descr, $when_command_is_over) = @_;

    my $w = ugtk2->new('');

    gtkadd(gtkset_size_request($w->{window}, 800, 400),
	   gtkpack_(Gtk2::VBox->new,
		    0, $descr,
		    1, my $log_scroll = create_scrolled_window(my $log_w = gtkset_editable(Gtk2::TextView->new, 0)),
		    0, my $close_w = Gtk2::Button->new(N("Close"))),
		   );
    my $stop_running = gtktext_get_log($command, $log_w, $log_scroll, $when_command_is_over);
    $close_w->signal_connect(clicked => sub { $stop_running->() and $w->destroy });
    $close_w->grab_focus;
    $w->show;
}

sub action_run {
    my ($selection) = @_;

    my @modes = ('gexec', 'rshp2');
    my $mode = $modes[0];

    my $w = ugtk2->new('');

    my $stop_running;
    gtkadd(gtkset_size_request($w->{window}, 500, 300),
	   gtkpack_(Gtk2::VBox->new,
		    1, gtkpack_(Gtk2::HBox->new,
				1, gtkpack_(Gtk2::VBox->new,
					    0, N("Execute %s", selection_to_string_on($selection)),
					    0, my $cmd_w = Gtk2::Entry->new,
					    1, my $log_scroll = create_scrolled_window(my $log_w = gtkset_editable(Gtk2::TextView->new, 0))),
				0, gtkpack_(Gtk2::VBox->new,
					    0, gtkpack(Gtk2::VBox->new,
						       my @modes_w = gtkradio($mode, @modes)),
					    0, my $verbose_w = Gtk2::CheckButton->new('verbose'))),
		    0, gtkpack(create_hbox(),
			       my $run_button = Gtk2::Button->new(N("Run")),
			       gtksignal_connect(Gtk2::Button->new(N("Close")), clicked => sub { !$stop_running || $stop_running->() and $w->destroy })),
		   ));

    my $verbose;
    $verbose_w->signal_connect(clicked => sub { $verbose = $verbose_w->get_active });

    mapn {
	my ($txt, $w) = @_;
	$w->signal_connect(clicked => sub { $mode = $txt });
    } \@modes, \@modes_w;

    my $do_action_run = sub {
	my $command = interface_cluster::cmd_to_exec($mode, $selection, $verbose, $cmd_w->get_text);
	$stop_running = gtktext_get_log($command, $log_w, $log_scroll);
    };

    $run_button->signal_connect(clicked => $do_action_run);

    $cmd_w->signal_connect(key_press_event => sub { 
	my (undef, $event) = @_;
	if ($event->keyval == $Gtk2::Gdk::Keysyms{Return}) {
	    $do_action_run->();
	    1;
	} else {
	    0;
	}
    });

    $w->show;
}

1;
