package drakcluster::nodes;

use strict;

use common;
use interface_cluster;

my @columns_keys = qw(name oar dhcp cpu compute admin group state);
my %columns_title = (
    name => N_("Node\nname"),
    oar => N_("OAR\nstate"),
    dhcp => N_("Known\nin Dhcp"),
    cpu => N_("Cpu\nnb"),
    compute => N_("Compute"),
    admin => N_("Admin"),
    group => N_("Urpmi\ngroup"),
    state => N_("LoadAvg\n(1|5|15) (User|System|Idle) (%d/%d)"),
);

sub create {
    my ($nodes) = @_;
    my $tree_view = create_tree_view(create_tree_model($nodes));
    create_titles($tree_view);
    update_nodes_up($tree_view, $nodes);
    set_titles($tree_view, '');
    $tree_view;
}

sub create_tree_model {
    my ($nodes) = @_;
    my $model = Gtk2::ListStore->new((map { $_ eq 'compute' || $_ eq 'admin' ? "Glib::Int" : "Glib::String" } @columns_keys), "Glib::String");

    foreach my $node (values %$nodes) {
	$model->append_set([ map_index { $::i => $node->{$_} } @columns_keys ]);
    }
    $model;
}

sub create_tree_view {
    my ($model) = @_;
    my $tree_view = Gtk2::TreeView->new;
    $tree_view->set_model($model);
    $tree_view->get_selection->set_mode('multiple');
    $tree_view->set_rules_hint(1);
    $tree_view;
}

sub create_titles {
    my ($tree_view) = @_;
    each_index {
      my $column = Gtk2::TreeViewColumn->new_with_attributes(undef, $_ eq 'compute' || $_ eq 'admin' ?  (Gtk2::CellRendererToggle->new, 'active') : ($_ eq 'dhcp' || $_ eq 'oar' ? (Gtk2::CellRendererPixbuf->new, 'stock-id') : (Gtk2::CellRendererText->new, 'text')) => $::i);
#      my $column = Gtk2::TreeViewColumn->new_with_attributes(undef, $_ eq 'compute' || $_ eq 'admin' ?  (Gtk2::CellRendererToggle->new, 'active') : ($_ eq 'dhcp' ? (Gtk2::CellRendererPixbuf->new, 'stock-id') : (Gtk2::CellRendererText->new, 'text')) => $::i);
      $tree_view->append_column($column);
      $column->set_sort_column_id($::i);
      $column->set_widget(my $label = Gtk2::Label->new);
      $column->clicked if $_ eq 'name'; # sort on "name" my default
      $label->show;
    } @columns_keys;
#	get_node_in(dhcp);
    #- add an empty column
    $tree_view->append_column(Gtk2::TreeViewColumn->new_with_attributes('', Gtk2::CellRendererText->new, 'text' => int @columns_keys));
}


my ($nb_total, $nb_up, $prev_title);
sub update_nodes_up {
    my ($tree_view, $nodes) = @_;

    my $state_column_indice if 0;
    my $dhcp_column_indice if 0;
    my $oar_column_indice if 0;
    $state_column_indice ||= find_index { $_ eq 'state' } @columns_keys;
    $dhcp_column_indice ||= find_index { $_ eq 'dhcp' } @columns_keys;
    $oar_column_indice ||= find_index { $_ eq 'oar' } @columns_keys;

#    my %updated = map { if_(1 || !$_->{state}, $_ => int(rand(3)) ? -1 : 1) } keys %$nodes; # -1 is down, 0 is unknown, 1 is up
    my %updated = interface_cluster::get_states($nodes); # -1 is down, 0 is unknown, 1 is up
    my %present_dhcp = interface_cluster::get_node_in($nodes, 'dhcp');
    #my %present_oar = interface_cluster::get_node_in($nodes, 'oar');
    my %present_oar = interface_cluster::get_oar_node_state($nodes);
    
    foreach (keys %present_dhcp) { $nodes->{$_}{dhcp}; }
    foreach (keys %present_oar) { $nodes->{$_}{oar}; }

    my $changed;
    foreach (keys %updated) {
	my $prev = $nodes->{$_}{state};
	my $new = $updated{$_};
	if ($prev != $new) {
	  $changed = 1;
	  if (!$prev) {
	    $nb_total++;
	    $nb_up += $new == 1 ? 1 : 0;
	  } else {
	    $nb_up += $new == 1 ? 1 : -1;
	  }
	  $nodes->{$_}{state} = $new;
	} else {
	  # force update to get last loadavg
#	  sleep(30);
	  $changed = 1;
	}
      }
    if ($changed) {
	my $model = $tree_view->get_model;
	tree_model_each_iter($model, [ keys %updated ],
			     sub { 
				 my ($val, $iter) = @_;
				 my $d = cluster_serverconf::dns_data()->{DOMAINNAME};
				 $model->set($iter, $state_column_indice => $updated{$val} == 1 ? get_loadavg("$val.$d") : N("Down"));
				 $model->set($iter, $dhcp_column_indice => $present_dhcp{$val} == 1 ? 'gtk-yes' : 'gtk-media-record');
				 my $state;
				 if ($present_oar{$val} =~ /Alive/) {
					 $state = "gtk-yes";
				 } elsif ($present_oar{$val} =~ /Absent/) {
					 $state = "gtk-disconnect";
				 } elsif ($present_oar{$val} =~ /Dead/) {
					 $state = "gtk-dialog-error";
				 } elsif ($present_oar{$val} =~ /Suspected/) {
					 $state = "gtk-dialog-warning";
				 } else {
					 $state = "gtk-dialog-question";
				 }
				 $model->set($iter, $oar_column_indice => $state);
#				 $model->set($iter, $oar_column_indice => $present_oar{$val} == 1 ? 'gtk-apply' : 'gtk-cancel');
#				 $model->set($iter, [ $state_column_indice => $updated{$val} == 1 ? N("up") : N("Down") ]);
			     });
	set_titles($tree_view, $prev_title);
    }
}

sub set_titles {
    my ($tree_view, $name) = @_;
    $prev_title = $name;
    each_index {
	my $title = sprintf(translate($columns_title{$_}), $nb_up, $nb_total);
	$tree_view->get_column($::i)->get_widget->set_markup($name eq $_ ? qq(<span foreground="blue">$title</span>) : $title);
    } @columns_keys;
}

sub tree_model_each_iter {
    my ($tree_model, $wanted_list, $f) = @_;
    my %wanted = map { $_ => 1 } @$wanted_list;
    $tree_model->foreach(sub {
	my ($model, $_path, $iter) = @_;
	my $val = $model->get($iter, 0);
	$f->($val, $iter) if $wanted{$val};
    }, undef);
}

sub get_selected {
    my ($tree_view) = @_;
#    my ($model, @selected) = $tree_view->get_selection->get_selected_rows;
    my @l;
    $tree_view->get_selection->selected_foreach(sub { 
	my ($model, $_path, $iter) = @_;
	push @l, $model->get($iter, 0);
    }, undef);
    @l;
}

sub get_selected_iters {
    my ($tree_view) = @_;
    my @l;
    $tree_view->get_selection->selected_foreach(sub { push @l, $_[2] }, undef);
    @l;
}

sub get_name_from_path {
    my ($model, $path) = @_;
    $model->get($model->get_iter($path), 0);
}
sub get_column_name_from_widget {
    my ($tree_view, $column) = @_;
    $columns_keys[find_index { $tree_view->get_column($_) == $column } 0 .. $#columns_keys+1];
}

sub get_current_and_column {
    my ($tree_view) = @_;
    my ($path, $column) = $tree_view->get_cursor;
    $path && get_name_from_path($tree_view->get_model, $path),
      get_column_name_from_widget($tree_view, $column);
}

sub handle_selection_change {
    my ($tree_view, $nodes, $selection, $buttons_set_sensitive, $popup_menus, $update_users_notebook) = @_;
    my $forget_it;
    $tree_view->get_selection->signal_connect(changed => sub {
	return if $forget_it;
	my ($gtkselection) = @_;
	my ($current, $column_chosen) = get_current_and_column($tree_view);
	my $val = $nodes->{$current}{$column_chosen};
	if ($column_chosen eq 'name') {
	} elsif ($column_chosen eq 'compute') {
	  my $column_indice = find_index { $_ eq $column_chosen } @columns_keys;
	  tree_model_each_iter($tree_view->get_model, [ $current ],
			       sub {
				     my ($_val, $iter) = @_;
				     $nodes->{$current}{$column_chosen} = !$val;
				     $tree_view->get_model->set($iter, $column_indice => !$val);
				     my $model = $tree_view->get_model;
				     my $new_val = !$model->get($iter, $column_indice); ;
				     $nodes->{$current}{$val} = $new_val;
				     my $comment = $new_val ? N("Desactive '%s' in Compute cluster", $current) : N("Active '%s' in Compute cluster", $current);
				     warn  $comment . ".\n";
				     my $_w = $::in->wait_message('', $comment . '...');
				     interface_cluster::chg_node_active_compute($current, !$val);
				   });
	} elsif ($column_chosen eq 'admin') {
	  my $column_indice = find_index { $_ eq $column_chosen } @columns_keys;
	  tree_model_each_iter($tree_view->get_model, [ $current ],
			       sub {
				     my ($_val, $iter) = @_;
				     $nodes->{$current}{$column_chosen} = !$val;
				     $tree_view->get_model->set($iter, $column_indice => !$val);
				     my $model = $tree_view->get_model;
				     my $new_val = !$model->get($iter, $column_indice); ;
				     $nodes->{$current}{$val} = $new_val;
				     my $comment = $new_val ? N("Desactive '%s' in Admin cluster", $current) : N("Active '%s' in Admin cluster", $current);
				     warn  $comment . ".\n";
				     my $_w = $::in->wait_message('', $comment . '...');
				     interface_cluster::chg_node_active_admin($current, !$val);
				   });
	} else {
	    $forget_it = 1;
	    my $select_or_not = member($current, get_selected($tree_view)) ? 'select_iter' : 'unselect_iter';
	    tree_model_each_iter($tree_view->get_model,
				 [ grep { $nodes->{$_}{$column_chosen} eq $val } keys %$nodes ], 
				 sub { $gtkselection->$select_or_not($_[1]) });
	    $forget_it = 0;
	}
	@$selection = map { $nodes->{$_} } get_selected($tree_view);
	set_titles($tree_view, member($column_chosen, 'partition', 'group') && $column_chosen);

	$_->(if_(@$selection, 'has_any'), if_($column_chosen eq 'group', 'has_group')) foreach @$buttons_set_sensitive;

    });

    $tree_view->signal_connect(button_press_event => sub { 
	my (undef, $event) = @_;
	$event->button eq '3' or return 0;

	my ($path, $column) = $tree_view->get_path_at_pos($event->x, $event->y);
	my $current = $path && get_name_from_path($tree_view->get_model, $path);
	$column or return 0;
	my $column_chosen = get_column_name_from_widget($tree_view, $column);

	my @selected = get_selected($tree_view);
	if ($popup_menus->{$column_chosen} && member($current, @selected)) {
	    $popup_menus->{$column_chosen}->($nodes, $event->time, \@selected, $nodes->{$current}{$column_chosen},
		sub {
		    my ($action, @vals) = @_; $action or return;
		    my $column_indice = find_index { $_ eq $column_chosen } @columns_keys;
		    $forget_it = 1;

		    if ($action eq 'add') {
#			my $iter; # = Gtk2::TreeIter->new;
			my $all_nodes = interface_cluster::get_node_list();
			foreach my $new_node (values %$all_nodes) {
			    $nodes->{$new_node->{name}} ||= do {
#				$tree_view->get_model->append($iter);
				$tree_view->get_model->append_set([map_index { $::i => $new_node->{$_} } @columns_keys]);
				$new_node;
			    };
			}
		      } elsif ($action eq 'change partitions') {
			$update_users_notebook->();
		      } else {
			foreach my $node (@selected) {
			    my $str;
			    $tree_view->get_model->foreach(sub { 
				my ($model, $_path, $iter) = @_;
				$str = $_[1]->to_string if $node eq $model->get($iter, 0);
			    }, undef);
			    my $iter = $tree_view->get_model->get_iter_from_string($str);

			    if ($action eq 'change') {
				$nodes->{$node}{$column_chosen} = $vals[0];
				$tree_view->get_model->set($iter, $column_indice => $vals[0]);
			    } elsif ($action eq 'remove') {
				delete $nodes->{$node};
				$tree_view->get_model->remove($iter);
			    }
			}
		    }
		    $forget_it = 0;
		});
	}
	1;
    });

}

1;

