package drakcluster::pxe_conf;

use strict;
use pxe_server_cluster;
use cluster_serverconf;
use common;
use mygtk2 qw(gtknew);
use ugtk2 qw(:wrappers :create);

use network::pxe;
my $config = "/var/lib/tftpboot/X86PC/linux/pxelinux.cfg/default";
my $help = "/var/lib/tftpboot/X86PC/linux/help.txt";
my $pxelinux_conf;

sub read() {
    { pxe_label => pxe_server_cluster::getDefaultLabel() };
}

sub done_pxe {
    my ($conf) = @_;
    pxe_server_cluster::setDefaultLabel($conf->{pxe_label});
}

sub create {
    my ($conf) = @_;
    my ($combo, $apply, $editconf);
    my ($ignore);
    map { $_ = Gtk2::Combo->new } $combo;

    $combo->set_popdown_strings(@{ pxe_server_cluster::get_pxe_labels() });
    $combo->entry->set_text($conf->{pxe_label});
    $combo->entry->signal_connect(changed => sub {
				    return if $ignore;
				    $apply->set_sensitive(1);
				    $conf->{pxe_label} = $combo->entry->get_text;
				  });

    map { 
	$_->set_use_arrows_always(1);
	$_->disable_activate;
	} $combo;

    $apply = Gtk2::Button->new(N("Set default PXE boot image"));
    $apply->signal_connect(clicked => sub {
			     $apply->set_sensitive(0);
			     done_pxe($conf);
			   });


    $editconf = Gtk2::Button->new(N("edit PXE conf"));
    $editconf->signal_connect(clicked => sub {
	system("xterm -e vim $config");
	update_treeview($config);
	$combo->set_popdown_strings(@{ pxe_server_cluster::get_pxe_labels() });
        done_pxe($conf);
    });

    map { $_->set_sensitive(0) } $apply;

    my $rebuild;
    $rebuild = Gtk2::Button->new(N("Recreate the default PXE configuration file"));
    $rebuild->set_sensitive(1);
    $rebuild->signal_connect(clicked => sub {
			       $::in->ask_yesorno('', N("Are you sure you want to recreate the default PXE configuration file?")) or return;
				 pxe_server_cluster::rebuild_img();
			     });


    my $buttonk;
    $buttonk = Gtk2::Button->new(N("Boot Parameters Kernel documentation"));
    my $file = "/usr/src/linux/Documentation/kernel-parameters.txt";
    if (-e $file) { $buttonk->set_sensitive(1) } else { $buttonk->set_sensitive(0) }
    $buttonk->signal_connect(clicked => sub {
	my $buffer = Gtk2::TextBuffer->new();
	my $iter = $buffer->get_start_iter;
	my $data = cat_($file);
	$buffer->insert($iter, $data);
	my $text = Gtk2::TextView->new_with_buffer($buffer);
	$text->set_editable(0);
	my $dialog = _create_dialog();
        $dialog->set_transient_for($::main_window);
        $dialog->set_title(N("Boot Parameters"));
        $dialog->set_modal(1);
        gtkpack_($dialog->vbox,
		 1, gtkset_size_request(create_scrolled_window($text), 570, 500),
		 1, gtksignal_connect(Gtk2::Button->new(N("Close")), clicked => sub {
		     $dialog->destroy;
		 }
				      ),
		 );
	$dialog->show_all;
    });


our $model = create_model();
our $treeview = Gtk2::TreeView->new_with_model($model);

use constant COLUMN_LABEL => 0;
use constant COLUMN_INFO => 1;
use constant COLUMN_KERNEL => 2;
use constant COLUMN_INITRD => 3;
use constant COLUMN_METHOD => 4;
use constant COLUMN_INTERFACE => 5;
use constant COLUMN_NETWORK => 6;
use constant COLUMN_SERVER => 7;
use constant COLUMN_DIRECTORY => 8;
use constant COLUMN_RAMDISK_SIZE => 9;
use constant COLUMN_VGA => 10;
use constant COLUMN_DISPLAY => 11;
use constant COLUMN_OTHERS => 12;
use constant COLUMN_AUTO_INSTALL => 13;
use constant NUM_COLUMNS => 14;

sub set_pxelinux_entry_at_iter {
    my ($model, $iter, $entry) = @_;
    $model->set($iter,
		COLUMN_LABEL, $entry->{label},
		COLUMN_INFO, $entry->{info},
		COLUMN_KERNEL, $entry->{kernel},
		COLUMN_INITRD, $entry->{initrd},
		COLUMN_METHOD, $entry->{method},
		COLUMN_INTERFACE, $entry->{interface},
		COLUMN_NETWORK, $entry->{network},
		COLUMN_SERVER, $entry->{server},
		COLUMN_DIRECTORY, $entry->{directory},
		COLUMN_RAMDISK_SIZE, $entry->{ramdisk_size},
		COLUMN_VGA, $network::pxe::vga_bios_to_resolution{$entry->{vga}},
		COLUMN_DISPLAY, $entry->{display},
		COLUMN_OTHERS, $entry->{others},
                COLUMN_AUTO_INSTALL, $entry->{auto_install},
		);
}


sub create_model() {
    my $model = Gtk2::ListStore->new("Glib::String", "Glib::String",  "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String", "Glib::String");
    set_pxelinux_entry_at_iter($model, $model->append, $_) foreach @{$pxelinux_conf->{entries}};
    return $model;
}

sub add_columns {
    my $treeview = shift;
    my $model = $treeview->get_model;
    each_index {
	my $renderer = Gtk2::CellRendererText->new;
	$renderer->set(editable => 0);
	$renderer->set_data(column => $::i);
	$treeview->insert_column_with_attributes(-1, $_, $renderer, 'text' => $::i);
    } N("Label"), N("Entry description"), N("Kernel image"), N("Initrd image"), N("Installation method"), N("Network interface"), N("DHCP or IP address"), N("Remote server name"), N("Remote installation directory"), N("Ramdisk size"), N("Frame buffer resolution"), N("Remote IP of X server"), N("Custom options"), "Auto_install";
}



sub update_treeview {
	my ($conf) = @_;
    my $pxelinux_conf = network::pxe::read_pxelinux_conf($conf, $help);
    $model->clear;                
    set_pxelinux_entry_at_iter($model, $model->append, $_) foreach @{$pxelinux_conf->{entries}};
}      

add_columns($treeview);
update_treeview($config);

my $defaultL = Gtk2::Label->new();
$defaultL->set_markup('<span foreground="BLUE"><b>Default boot:</b></span>');
gtkpack_(Gtk2::VBox->new(0, 0),
	 1, gtknew('ScrolledWindow', width => 500, height => 300, child => $treeview),
	 0, Gtk2::HSeparator->new,
	 0, gtkpack_(Gtk2::HBox->new(0, 0),
		     1, $defaultL,
		     1, $combo,
		     1, $apply,
		     ),
	 0, Gtk2::HSeparator->new,
	 0, gtkpack_(Gtk2::HBox->new(0, 0),
		     1, $buttonk,
		     1, $editconf,
		     0, $rebuild,
		     ),
	 ),

sub {
    $apply->get_sensitive and return; # modified but not applied, keeping modification
    put_in_hash($conf, &read);
    $ignore = 1;
    $combo->entry->set_text($conf->{pxe_label});
    $ignore = 0;
};
}

1;
