package drakcluster::server_conf;

use strict;
use common;

sub read() {
    my %main = getVarsFromSh("$::prefix/etc/clusterserver.conf");
    \%main;
}

sub done {
    my ($conf, $network_interfaces) = @_;
    setVarsInSh("$::prefix/etc/clusterserver.conf", $conf);
}

sub fields {
    my ($conf, $network_interfaces, $default_conf) = @_;
    my $current_conf = { %$conf };
    add2hash($conf, $default_conf);
   my @l = (
        [ NODENAME => 'Node name (Administration network)' , check => \&check_non_empty ],

        [ STARTNODE => 'IP of the first node', check => \&check_number ],
        [ FINISHNODE => 'IP of the last node', check => sub { &check_number || $conf->{STARTNODE} >= $conf->{FINISHNODE} && N("\"The First node\" must have an IP  lower than \"the last node\"") } ],

        [ ADMIN_INTERFACE => 'Administration Network Interface', list => $network_interfaces ],
        [ EXTERNAL_INTERFACE => 'External Network Interface', list => $network_interfaces ],
        [ NTPSERVER => N("NTP Server IP Address"), check => sub { &check_ip || check_default(@_, $default_conf->{NTPSERVER}, $current_conf->{NTPSERVER}) } ],
        [ DOMAINNAME => N("Administration node domain name"), check => sub { check_default(@_, $default_conf->{DOMAINNAME}, $current_conf->{DOMAINNAME}) } ],
        [ NISDOMAINNAME => N("NIS domain name"), check => sub { check_default(@_, $default_conf->{NISDOMAINNAME}, $current_conf->{NISDOMAINNAME}) } ],
	[ LDAPSERVER => N("LDAP server"), check => sub { check_default(@_, $default_conf->{LDAPSERVER}, $current_conf->{LDAPSERVER}) } ],
	[ LDAPDOMAIN => N("LDAP domain"), check => sub { check_default(@_, $default_conf->{LDAPDOMAIN}, $current_conf->{LDAPDOMAIN}) } ],
        [ IPOFFORWARDER => 'IP address of your external DNS', check => sub { $_[0] && check_ip(@_) } ],
        [ ADDSEARCH => 'ADDSEARCH ' . N("Domain name of your external network") ],
#	[ ORIGIN => 'ORIGIN ' . N("Domain that locally posted mails appears to come from") ],
#	[ CDROM => 'CDROM where is your cdrom', check => sub { check_default(@_, $default_conf->{CDROM}, $current_conf->{CDROM}) } ],    );
	);

#        [ DNSKEY => 'DNS server dnskey', check => \&check_non_empty ],

    map { 
	my ($field, $label, %opts) = @$_;
	$opts{val} = \$conf->{$field};
	$opts{label} = $label;
	\%opts;
    } @l;
}

sub check_field {
    my ($in, $field) = @_;
    warn "checking $field->{label}\n";
    $field->{check} or return 1;
    my $err = $field->{check}->(${$field->{val}}, $field->{label}, warn => $in);
    $in->ask_warn('', $err) if $err && $err ne 'already_displayed';
    !$err;
}


# CHECKERS #####################################################################
sub is_ip {
    $::isInstall ? &network::network::is_ip : &cluster_fonction_common::is_ip;
}

sub check_non_empty {
    my ($val, $txt) = @_;
    $val eq '' && N("Field \"%s\" must not be empty", $txt);
}
sub check_number {
    my ($val, $txt) = @_;
    $val !~ /^\d+$/ && N("Field \"%s\" must be a number", $txt);
}
sub check_ip {
    my ($val, $txt) = @_;
    !is_ip($val) && N("Field \"%s\" must be an IP", $txt);
}

sub check_default {
    my ($val, $txt, $warn, $in, $default_val, $pre_val) = @_;
    if ($default_val && $val ne $default_val && $val ne $pre_val) {
	if ($warn eq 'warn') {
	    return '' if $in->ask_okcancel('', N("You changed the default value of field \"%s\". Are you sure you want to do this?", $txt));
	}
	'already_displayed';
    } else {
	'';
    }
}
1;
