package drakcluster::users;

use strict;

use common;
use ugtk2 qw(:wrappers :create);


sub create {
    my ($box, $users, $partitions) = @_;
    $_->destroy foreach $box->get_children;
    my $tree_view = create_tree_view(create_tree_model($users, $partitions));
    create_titles($tree_view, $partitions, handle_change($tree_view, $users, $partitions));
    gtkadd($box, gtkset_size_request(create_scrolled_window($tree_view), 300, 400));

    my $selection; my @actions;  
    my $actions_box = Gtk2::HButtonBox->new;
    my @buttons_set_sensitive = map {
      my ($descr, $flag, $f) = @$_;
      my $button = Gtk2::Button->new($descr);
      $button->signal_connect(clicked => sub { $f->($selection, $descr) });
      gtkadd($actions_box, $button);
      sub { $button->set_sensitive(member($flag, @_)) }
    } @actions;
    $actions_box, @buttons_set_sensitive;
}

sub create_tree_model {
    my ($users, $partitions) = @_;
    my $model = Gtk2::ListStore->new("Glib::String", (map { "Glib::Int" } @$partitions), "Glib::String");
    update_users($model, $users, $partitions);
    $SIG{USR1} = sub {
	my $new = interface_cluster::get_users_list();
	%$users = %$new;
	update_users($model, $users, $partitions);
    };
    $model;
}

sub update_users {
    my ($model, $users, $partitions) = @_;

    $model->clear;
    foreach my $user (keys %$users) {
	$model->append_set( [ 0 => $user, map_index { $::i + 1 => $users->{$user}{$_} } @$partitions ]);
    }
    $model;
}

sub create_tree_view {
    my ($model) = @_;
    my $tree_view = Gtk2::TreeView->new;
    $tree_view->set_model($model);
    $tree_view->set_rules_hint(1);
    $tree_view;
}

sub create_titles {
    my ($tree_view, $partitions, $handle_change) = @_;

    my $column = Gtk2::TreeViewColumn->new_with_attributes(N("User name"), Gtk2::CellRendererText->new, 'text' => 0);
    $column->set_sort_column_id(0);
    $column->set_resizable(1);
    $tree_view->append_column($column);
    $column->clicked; # sort on user name by default

    each_index {
	my $i = $::i + 1;
	my $column = Gtk2::TreeViewColumn->new_with_attributes($_, my $renderer = Gtk2::CellRendererToggle->new, 'active' => $i);
	$column->set_sort_column_id($i);
	$renderer->signal_connect(toggled => sub { $handle_change->($i, @_) });
	$tree_view->append_column($column);
    } @$partitions;

    $tree_view->append_column(Gtk2::TreeViewColumn->new_with_attributes('', Gtk2::CellRendererText->new, 'text' => @$partitions + 1));
}

sub handle_change {
    my ($tree_view, $users, $partitions) = @_;
    my $model = $tree_view->get_model;
    sub {
	my ($column_nb, undef, $path_str) = @_;
	my $iter = $model->get_iter_from_string($path_str);
	my $user = $model->get($iter, 0);
	my $new_val = !$model->get($iter, $column_nb);
	my $partition = $partitions->[$column_nb - 1];

	$model->set($iter, $column_nb => $new_val);
	$users->{$user}{$partition} = $new_val;
	my $comment = $new_val ? N("Add user '%s' in computing partition '%s'", $user, $partition) : N("Remove user '%s' from computing partition '%s'", $user, $partition);
	warn  $comment . ".\n";
	my $_w = $::in->wait_message('', $comment . '...');
	interface_cluster::chg_user_part($partition, $user, $new_val);
    };
}

1;

